/*	engine_position_isrs.c

	Copyright 2008 Fred Cooke

	This file is part of the FreeEMS project.

	FreeEMS software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	FreeEMS software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any FreeEMS software.  If not, see <http://www.gnu.org/licenses/>.

	We ask that if you make any changes to this file you send them upstream to us at admin@diyefi.org

	Thank you for choosing FreeEMS to run your engine! */

#include "engine_position_isrs.h"

/* TODO Summary of intended engine position capture scheme
 * 
 * Position/RPM signal interpretation :
 * Discard edges that have arrived too soon (lose sync here?)
 * Check to ensure we haven't lost sync (pulse arrives too late)
 * Compare time stamps of successive edges and calculate RPM
 * Store RPM and position in globals
 * 
 * Schedule events :
 * loop through all events (spark and fuel), schedule those that fall sufficiently after this tooth and before the next one we expect.
 * 
 * Sample ADCs :
 * Grab a unified set of ADC readings at one time in a consistent crank location to eliminate engine cycle dependent noise.
 * Set flag stating that New pulse, advance, etc should be calculated.
 * 
 */

void PrimaryRPMISR(void)
{
	/* Clear the interrupt flag for this input compare channel */
	TFLG = 0x01;

	// don't run until the second trigger has come in (VERY temporary)
	if(!(coreStatusA & PRIMARY_SYNC)){
		return;
	}

	/* Save all relevant available data here */
	unsigned short edgeTimeStamp = TC0;			/* Save the timestamp */
	unsigned char PTITCurrentState = PTIT;		/* Save the values on port T regardless of the state of DDRT */
//	unsigned char PORTBCurrentState = PORTB;	/* Save ignition output state for first bank */
//	unsigned char PORTACurrentState = PORTA;	/* Save ignition output state for second bank */
	
	if(!(PTITCurrentState & 0x01)){
		// echo input condition to output LED adjacent on JimStim
		PORTJ |= 0x80;

		// increment crank pulses
		primaryPulsesPerSecondaryPulse++;
		
		// temporary data from inputs
		primaryLeadingEdgeTimeStamp = edgeTimeStamp;
		timeBetweenSuccessivePrimaryPulses = lastPrimaryPulseTimeStamp - primaryLeadingEdgeTimeStamp;
		lastPrimaryPulseTimeStamp = primaryLeadingEdgeTimeStamp;
		timeBetweenSuccessivePrimaryPulsesBuffer = (timeBetweenSuccessivePrimaryPulses >> 1) + (timeBetweenSuccessivePrimaryPulsesBuffer >> 1);
		
		// TODO also schedule ignition (both start and end if appropriate with the timing involved.)
		
		// TODO make scheduling either fixed from boot with a limited range, OR preferrably if its practical scheduled on the fly to allow arbitrary advance and retard of both fuel and ignition.
		
//		short modulusGenerator = 12/ports; TODO make it cyl count generic
//		if((primaryPulsesPerSecondaryPulse % modulusGenerator) == 0){
		if(((primaryPulsesPerSecondaryPulse % 4) == 0) && (totalPulseWidth > injectorMinimumPulseWidth)){ // this needs to be per channel. TODO
			unsigned short channel = (primaryPulsesPerSecondaryPulse / 4) - 1;
			//	make sure pulsewidth never ends later than start- isrturnoffexecutiontime
			unsigned short starttime = primaryLeadingEdgeTimeStamp + trailingEdgeSecondaryRPMInputCodeTime + totalAngleAfterReferenceInjection; // TODO use individual trims
			
			/* Setup the banked pulse widths array offset */
			unsigned char bankOffset;
			if(readingBank){ // TODO is this right??
				bankOffset = 6;
			}else{
				bankOffset = 0;
			}

			// schedule the appropriate channel
			if(!(*injectorMainControlRegisters[channel] & injectorMainEnableMasks[channel]) || (rescheduleFuelFlags & injectorMainOnMasks[channel])){ /* If the timer isn't still running, or if its set too long, set it to start again at the right time soon */
				*injectorMainControlRegisters[channel] |= injectorMainEnableMasks[channel];
				*injectorMainTimeRegisters[channel] = starttime;
				TIE |= injectorMainOnMasks[channel];
				TFLG = injectorMainOnMasks[channel];
			}else{
				injectorMainStartTimesHolding[channel] = starttime;
				selfSetTimer |= injectorMainOnMasks[channel]; // setup a bit to let the timer interrupt know to set its own new start from a var
			}

			// TODO sample ADCs on teeth other than that used by the scheduler in order to minimise peak run time and get clean signals

			/* Read ADCs into the correct bank */
			iat[readingBank] = ATD0DR0;
			clt[readingBank] = ATD0DR1;
			tps[readingBank] = ATD0DR2;
			ego[readingBank] = ATD0DR3;
			map[readingBank] = ATD0DR4;
			spareADC5[readingBank] = ATD0DR5;
			spareADC6[readingBank] = ATD0DR6;
			spareADC7[readingBank] = ATD0DR7;

			// TODO average them here or maybe do that in main as well, and just miss them out if too slow?

//			newVal var word        ' the value from the ADC
//			smoothed var word    ' a nicely smoothed result 
//
//			if newval > smoothed then
//			        smoothed = smoothed + (newval - smoothed)/alpha
//			else
//			        smoothed = smoothed - (smoothed - newval)/alpha
//			endif
	
			// from : http://www.tigoe.net/pcomp/code/category/code/arduinowiring/41			

			/* Set flag to say calc required */
			coreStatusA |= CALC_FUEL_IGN;
			
			/* Reset the counter for reading timeout */
			readingCounter = 0;
		}
	}else{
		PORTJ &= 0x7F;
	}
	
	// suss out rpm and accurate TDC reference
	
	// if you say it quick, it doesn't sound like much :
	// schedule fuel and ign based on spark cut and fuel cut and timing vars and status vars config vars

	/* Clear the interrupt flag for this input compare channel */
//	TFLG = 0x01; WHY does this NOT work here???
}

void SecondaryRPMISR(void)
{
	/* Clear the interrupt flag for this input compare channel */
	TFLG = 0x02;

	/* Save all relevant available data here */
	unsigned short edgeTimeStamp = TC1;			/* Save the timestamp */
	unsigned char PTITCurrentState = PTIT;		/* Save the values on port T regardless of the state of DDRT */
//	unsigned char PORTBCurrentState = PORTB;	/* Save ignition output state for first bank */
//	unsigned char PORTACurrentState = PORTA;	/* Save ignition output state for second bank */
	
	if(!(PTITCurrentState & 0x02)){
		// echo input condition to output LED adjacent on JimStim
		PORTJ |= 0x40;
		
		// Because this is our only reference, each time we get this pulse, we know where we are at (simple mode so far)
		coreStatusA |= PRIMARY_SYNC;
		
		/* If we are on our second time round, reset the phase and pulses */ 
		if(coreStatusA & ENGINE_PHASE){
			
			// display the crank pulses
			PORTM = (char)primaryPulsesPerSecondaryPulseBuffer;
			
			primaryPulsesPerSecondaryPulseBuffer = primaryPulsesPerSecondaryPulse;
			primaryPulsesPerSecondaryPulse = 0;
			coreStatusA &= ~ENGINE_PHASE;
			
			// find out what our timer value means
			unsigned long currentExt;
			if(TFLGOF && !(edgeTimeStamp & 0x8000)){ /* see 10.3.5 paragraph 4 of 68hc11 ref manual for details */
				currentExt = timerExtension + 1; 
			}else{
				currentExt = timerExtension;
			}
			
			// make our time value 32 bit
			unsigned long currentTime = currentExt << 16; // leave the low word all zeros and move the timer extension to the high word
			currentTime += edgeTimeStamp; // replace the zeros in the low word with the contents of the timer capture
			
			// get the data we actually want
			engineCyclePeriod = currentTime - lastSecondaryOddTimeStamp; // save the period			
			lastSecondaryOddTimeStamp = currentTime; // save this stamp for next time round
		}else{
			coreStatusA |= ENGINE_PHASE;
		}
	}else{
		PORTJ &= 0xBF;
	}

	// suss out phase/engine cycle reference showing which bank we are on

	/* Clear the interrupt flag for this input compare channel */
//	TFLG = 0x02; WHY does this NOT work here?
}
