/*	ignition_isrs.c

	Copyright 2008 Fred Cooke

	This file is part of the FreeEMS project.

	FreeEMS software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	FreeEMS software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any FreeEMS software.  If not, see <http://www.gnu.org/licenses/>.

	We ask that if you make any changes to this file you send them upstream to us at admin@diyefi.org

	Thank you for choosing FreeEMS to run your engine! */

#include "ignition_isrs.h"

/* Summary of intended ignition timing scheme
 * 
 * Set TWO PIT timers based on a reference point (probably single cam trigger or optionally toggle bit based on missing tooth on crank)
 * Arm one PIT timer interrupt to trigger when we need to start dwelling
 * Arm the other PIT timer interrupt to trigger when we need to fire
 * Configure a variable to point to the next pin to turn on for dwell
 * Configure a variable to point to the next pin to turn off to fire
 * On PIT interrupt for dwell check to see that spark should have finished
 * If so, turn on coil to dwell
 * If not, reset dwell timer interrupt for end of spark event
 * On PIT interrupt for spark, turn off coil (amything else?)
 * Repeat for each cylinder. 
 */

/* Further consideration to spark duration and dwell starting possibly needs to be done. */

/* further reading : ftp://ftp-sop.inria.fr/esterel/pub/papers/FDL99-camready.pdf section 4.1 has a nice diagram */

void IgnitionDwellISR(void)
{
	// clear flag
	PITTF = 0x01;

	// start dwelling
//	PORTS_BA |= dwellStartMasks[activeDwellChannel];
	
	PORTS ^= 0x80;
	
	// turn off the int
	PITINTE &= 0xFE;
}
	

// TODO add in self schedule stuff for igntion timers, add in delay / latency corrections for that.



//	// start dwelling channels that need it
//	PORTS_BA |= dwellStartMask;
//	
//	// next channel
//	if(dwellStartMask & 0x8000){
//		dwellStartMask = 0x0001;
//	}else{
//		dwellStartMask = dwellStartMask < 1;
//	}
//	
//	PORTB ^= 0x01;
//	
//	// store the dwell in the load register
//	PITLD1 = dwellLength;
//	
//	// force load of dwell length to start timer counting now.
//	PITFLT = 0x02;
//	
//	// re enable turn off int
//	PITINTE |= 0x02;
//}

void IgnitionFireISR(void)
{
	// clear the flag
	PITTF = 0x02;

	// fire the coil
	//PORTS_BA &= ignitionMasks[activeIgnitionChannel];

	PORTS &= 0x40;
	
	// turn off the int
	PITINTE &= 0xFD;
}
	
//	// fire coil/stop dwell
//	PORTS_BA &= dwellEndMask;
//	
//	PORTB ^= 0x02;
//	
//	// shift to next cylinder
//	if(~dwellEndMask & 0x8000){
//		dwellEndMask = ~(~dwellEndMask < 1);
//	}else{
//		dwellEndMask = 0xFFFE;
//	}
//}
