/*	main.c

	Copyright 2008 Fred Cooke

	This file is part of the FreeEMS project.

	FreeEMS software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	FreeEMS software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any FreeEMS software.  If not, see <http://www.gnu.org/licenses/>.

	We ask that if you make any changes to this file you send them upstream to us at admin@diyefi.org

	Thank you for choosing FreeEMS to run your engine! */

#include "main.h"

/* Thanks to alex for inspiration :-) */
void inject(channel)
{
	if(!localFuelCut){
		PTT ^= BIT0 << channel;
		if(localStagedInjection){
			PORTK ^= BIT0 << channel;
		}
	}
	if(!localSparkCut){
		if(coreStatusA & ENGINE_PHASE){
			PORTA ^= BIT0 << channel;
		}else{
			PORTB ^= BIT0 << channel;
		}
	}
}

void adjustPWM(void)
{
	PWMDTY0 = ATD0DR0 >> 2; // scale raw adc to a duty
	PWMDTY1 = ATD0DR1 >> 2; // scale raw adc to a duty
	PWMDTY2 = ATD0DR2 >> 2; // scale raw adc to a duty
	PWMDTY3 = ATD0DR3 >> 2; // scale raw adc to a duty
	PWMDTY4 = ATD0DR4 >> 2; // scale raw adc to a duty
	PWMDTY5 = ATD0DR5 >> 2; // scale raw adc to a duty
	PWMDTY6 = ATD0DR6 >> 2; // scale raw adc to a duty
	//PWMDTY7 = ATD0DR7 >> 2; // scale raw adc to a duty (user led instead)
}

/* Based on the latest ADC readings, determine pulsewidth, dwell, and advance for all channels */
void calculateFuelAndIgnition(void)
{
	/* Setup the banked pulse widths array offset */
	unsigned char bankOffset;
	if(useThisBank){ // TODO is this right??
		bankOffset = 6;
	}else{
		bankOffset = 0;
	}

	/* "Calculate" the nominal total pulse width before per channel corrections */
	totalPulseWidth = (ego[useThisBank] << 6) + (map[useThisBank] >> 4);

	/* "Calculate" the individual fuel pulse widths */
	unsigned char channel;
	for(channel = 0; channel < INJECTION_CHANNELS; channel++){
		injectorMainPulseWidths[channel + bankOffset] = totalPulseWidth;
	}

	// TODO x 6 main, x 6 staged, x 6 flags for staged if(coreSettingsA & STAGED_ON){}
	
	/* Set the staged status on or off (for now based on changeable settings) */
	if(coreSettingsA & STAGED_ON){
		coreStatusA |= STAGED_REQUIRED;
	}else{
		coreStatusA &= ~STAGED_REQUIRED;
	}
	
	// temporary ign tests
	unsigned short intendedPeriod = 0x3F + (map[useThisBank] << 6);
	unsigned short intendedDwell = intendedPeriod >> 1;
	unsigned short minPeriod = ignitionMinimumDwell << 1;
	if(intendedDwell < ignitionMinimumDwell){
		dwellLength = ignitionMinimumDwell;
	}else{
		dwellLength = intendedDwell;
	}
	if(intendedPeriod < minPeriod){
		dwellPeriod = minPeriod;
	}else{
		dwellPeriod = intendedPeriod;
	}
	PITLD0 = dwellPeriod;
	
	/* Calculate the fuel advances */
	// TODO x 6
	// just use one for all for now
	totalAngleAfterReferenceInjection = (tps[useThisBank] << 6);
	
	/* Calculate the dwell period */
	// TODO x 1
	
	/* Calculate the ignition advances */
	//TODO x 12
}

int  main(){
	// Set everything up.
	init();
		
	// Run forever repeating.
	while(TRUE){
		/* If ADCs require forced sampling, sample now */
		if(coreStatusA & FORCE_READING){
			/* Read ADCs into the correct bank */			
			iat[readingBank] = ATD0DR0;
			clt[readingBank] = ATD0DR1;
			tps[readingBank] = ATD0DR2;
			ego[readingBank] = ATD0DR3;
			map[readingBank] = ATD0DR4;
			spareADC5[readingBank] = ATD0DR5;
			spareADC6[readingBank] = ATD0DR6;
			spareADC7[readingBank] = ATD0DR7;

			/* Set flag to say calc required */
			coreStatusA |= CALC_FUEL_IGN;
			
			/* Clear flag */
			coreStatusA &= ~FORCE_READING;
		}

		/* If required, do main fuel and ignition calcs first */
		if(coreStatusA & CALC_FUEL_IGN){
			/* clear the flag and switch banks together */
			ATOMIC_START();
			/* Clear the calc required flag */
			coreStatusA &= ~CALC_FUEL_IGN;
			
			/* Switch available banks so that we have a stable set of variables to read from */
			if(readingBank){
				readingBank = 0;
				useThisBank = 1;
			}else{
				readingBank = 1;
				useThisBank = 0;
			}
			ATOMIC_END();
			
			/* Perform the calculations TODO possibly move this to the software interrupt if it makes sense to do so */
			calculateFuelAndIgnition();
		}

		// watch power supply irregularities on port B
		//PORTB = lowVoltISRCounter;
		
		sleep(5);
		
		// on once per cycle for J0 (tacho)
		PORTJ ^= 0x01;

		// PWM experimentation
		adjustPWM();
	}
}
